@@ -20,14 +20,16 @@ if MYPY:
     from UM.Tool import Tool
 
 
-##  Glue class that holds the scene, (active) view(s), (active) tool(s) and possible user inputs.
-#
-#   The different types of views / tools / inputs are defined by plugins.
-#   \sa View
-#   \sa Tool
-#   \sa Scene
 @signalemitter
 class Controller:
+    """Glue class that holds the scene, (active) view(s), (active) tool(s) and possible user inputs.
+    
+    The different types of views / tools / inputs are defined by plugins.
+    :sa View
+    :sa Tool
+    :sa Scene
+    """
+
     def __init__(self, application: "Application") -> None:
         super().__init__()  # Call super to make multiple inheritance work.
 
@@ -55,9 +57,12 @@ class Controller:
         PluginRegistry.addType("tool", self.addTool)
         PluginRegistry.addType("input_device", self.addInputDevice)
 
-    ##  Add a view by name if it"s not already added.
-    #   \param view \type{View} The view to be added
     def addView(self, view: View) -> None:
+        """Add a view by name if it"s not already added.
+
+        :param view: The view to be added
+        """
+
         name = view.getId()
         if name not in self._views:
             self._views[name] = view
@@ -66,28 +71,40 @@ class Controller:
         else:
             Logger.log("w", "%s was already added to view list. Unable to add it again.", name)
 
-    ##  Request view by name. Returns None if no view is found.
-    #   \return View \type{View} if name was found, none otherwise.
     def getView(self, name: str) -> Optional[View]:
+        """Request view by name. Returns None if no view is found.
+
+        :return: View  if name was found, none otherwise.
+        """
+
         try:
             return self._views[name]
         except KeyError:  # No such view
             Logger.log("e", "Unable to find %s in view list", name)
             return None
 
-    ##  Return all views.
-    #   \return views \type{dict}
     def getAllViews(self) -> Dict[str, View]:
+        """Return all views.
+
+        :return: views
+        """
+
         return self._views
 
-    ##  Request active view. Returns None if there is no active view
-    #   \return view \type{View} if an view is active, None otherwise.
     def getActiveView(self) -> Optional[View]:
+        """Request active view. Returns None if there is no active view
+
+        :return: view if an view is active, None otherwise.
+        """
+
         return self._active_view
 
-    ##  Set the currently active view.
-    #   \param name \type{string} The name of the view to set as active
     def setActiveView(self, name: str) -> None:
+        """Set the currently active view.
+
+        :param name:  The name of the view to set as active
+        """
+
         Logger.log("d", "Setting active view to %s", name)
         try:
             if self._active_view:
@@ -104,44 +121,58 @@ class Controller:
         except Exception as e:
             Logger.logException("e", "An exception occurred while switching views: %s", str(e))
 
-    ##  Emitted when the list of views changes.
     viewsChanged = Signal()
+    """Emitted when the list of views changes."""
 
-    ##  Emitted when the active view changes.
     activeViewChanged = Signal()
+    """Emitted when the active view changes."""
 
-    ##  Add a stage by name if it's not already added.
-    #   \param name \type{string} Unique identifier of stage (usually the plugin name)
-    #   \param stage \type{Stage} The stage to be added
     def addStage(self, stage: Stage) -> None:
+        """Add a stage if it's not already added.
+
+        :param stage: The stage to be added
+        """
+
         name = stage.getId()
         if name not in self._stages:
             self._stages[name] = stage
             self.stagesChanged.emit()
 
-    ##  Request stage by name. Returns None if no stage is found.
-    #   \param name \type{string} Unique identifier of stage (usually the plugin name)
-    #   \return Stage \type{Stage} if name was found, none otherwise.
     def getStage(self, name: str) -> Optional[Stage]:
+        """Request stage by name. Returns None if no stage is found.
+
+        :param name: Unique identifier of stage (usually the plugin name)
+        :return: Stage if name was found, None otherwise.
+        """
+
         try:
             return self._stages[name]
         except KeyError:  # No such view
             Logger.log("e", "Unable to find %s in stage list", name)
             return None
 
-    ##  Return all stages.
-    #   \return stages \type{dict}
     def getAllStages(self) -> Dict[str, Stage]:
+        """Return all stages.
+
+        :return: stages
+        """
+
         return self._stages
 
-    ##  Request active stage. Returns None if there is no active stage
-    #   \return stage \type{Stage} if an stage is active, None otherwise.
     def getActiveStage(self) -> Optional[Stage]:
+        """Request active stage. Returns None if there is no active stage
+
+        :return: stage if an stage is active, None otherwise.
+        """
+
         return self._active_stage
 
-    ##  Set the currently active stage.
-    #   \param name \type{string} The name of the stage to set as active
     def setActiveStage(self, name: str) -> None:
+        """Set the currently active stage.
+
+        :param name: The name of the stage to set as active
+        """
+
         try:
             # Don't actually change the stage if it is the current selected one.
             if self._active_stage != self._stages[name]:
@@ -159,15 +190,18 @@ class Controller:
         except Exception as e:
             Logger.logException("e", "An exception occurred while switching stages: %s", str(e))
 
-    ##  Emitted when the list of stages changes.
     stagesChanged = Signal()
+    """Emitted when the list of stages changes."""
 
-    ##  Emitted when the active stage changes.
     activeStageChanged = Signal()
+    """Emitted when the active stage changes."""
 
-    ##  Add an input device (e.g. mouse, keyboard, etc) if it's not already added.
-    #   \param device The input device to be added
     def addInputDevice(self, device: InputDevice) -> None:
+        """Add an input device (e.g. mouse, keyboard, etc) if it's not already added.
+
+        :param device: The input device to be added
+        """
+
         name = device.getId()
         if name not in self._input_devices:
             self._input_devices[name] = device
@@ -175,53 +209,74 @@ class Controller:
         else:
             Logger.log("w", "%s was already added to input device list. Unable to add it again." % name)
 
-    ##  Request input device by name. Returns None if no device is found.
-    #   \param name \type{string} Unique identifier of input device (usually the plugin name)
-    #   \return input \type{InputDevice} device if name was found, none otherwise.
     def getInputDevice(self, name: str) -> Optional[InputDevice]:
+        """Request input device by name. Returns None if no device is found.
+
+        :param name: Unique identifier of input device (usually the plugin name)
+        :return: input device if name was found, none otherwise.
+        """
+
         try:
             return self._input_devices[name]
         except KeyError:  # No such device
             Logger.log("e", "Unable to find %s in input devices", name)
             return None
 
-    ##  Remove an input device from the list of input devices.
-    #   Does nothing if the input device is not in the list.
-    #   \param name \type{string} The name of the device to remove.
     def removeInputDevice(self, name: str) -> None:
+        """Remove an input device from the list of input devices.
+
+        Does nothing if the input device is not in the list.
+        :param name: The name of the device to remove.
+        """
+
         if name in self._input_devices:
             self._input_devices[name].event.disconnect(self.event)
             del self._input_devices[name]
 
-    ##  Request the current fallbacl tool.
-    #   \return Id of the fallback tool
     def getFallbackTool(self) -> str:
+        """Request the current fallbacl tool.
+
+        :return: Id of the fallback tool
+        """
+
         return self._fallback_tool
 
-    ##  Set the current active tool. The tool must be set by name.
-    #   \param tool The tools name which shall be used as fallback
     def setFallbackTool(self, tool: str) -> None:
+        """Set the current active tool. The tool must be set by name.
+
+        :param tool: The tools name which shall be used as fallback
+        """
+
         if self._fallback_tool is not tool:
             self._fallback_tool = tool
 
-    ##  Request tool by name. Returns None if no tool is found.
-    #   \param name \type{string} Unique identifier of tool (usually the plugin name)
-    #   \return tool \type{Tool} if name was found, None otherwise.
     def getTool(self, name: str) -> Optional["Tool"]:
+        """Request tool by name. Returns None if no tool is found.
+
+        :param name: Unique identifier of tool (usually the plugin name)
+        :return: tool if name was found, None otherwise.
+        """
+
         try:
             return self._tools[name]
         except KeyError:  # No such tool
             Logger.log("e", "Unable to find %s in tools", name)
             return None
 
-    ##  Get all tools
-    #   \return tools \type{dict}
     def getAllTools(self) -> Dict[str, "Tool"]:
+        """Get all tools
+
+        :return: tools
+        """
+
         return self._tools
 
-    ##  Add a Tool (transform object, translate object) if its not already added.
-    #   \param tool \type{Tool} Tool to be added
     def addTool(self, tool: "Tool") -> None:
+        """Add a Tool (transform object, translate object) if its not already added.
+
+        :param tool: Tool to be added
+        """
+
         name = tool.getId()
         if name not in self._tools:
             self._tools[name] = tool
@@ -241,20 +296,29 @@ class Controller:
             self._tool_operation_active = False
             self.toolOperationStopped.emit(tool)
 
-    ##  Gets whether a tool is currently in use
-    #   \return \type{bool} true if a tool current being used.
     def isToolOperationActive(self) -> bool:
+        """Gets whether a tool is currently in use
+
+        :return: true if a tool current being used.
+        """
+
         return self._tool_operation_active
 
-    ##  Request active tool. Returns None if there is no active tool
-    #   \return Tool if a tool is active, None otherwise.
     def getActiveTool(self) -> Optional["Tool"]:
+        """Request active tool. Returns None if there is no active tool
+
+        :return: Tool if a tool is active, None otherwise.
+        """
+
         return self._active_tool
 
-    ##  Set the current active tool.
-    #   The tool can be set by name of the tool or directly passing the tool object.
-    #   \param tool A tool object or the name of a tool.
     def setActiveTool(self, tool: Optional[Union["Tool", str]]):
+        """Set the current active tool.
+
+        The tool can be set by name of the tool or directly passing the tool object.
+        :param tool: A tool object or the name of a tool.
+        """
+
         from UM.Tool import Tool
         if self._active_tool:
             self._active_tool.event(ToolEvent(ToolEvent.ToolDeactivateEvent))
@@ -279,45 +343,54 @@ class Controller:
                 self._active_tool.event(ToolEvent(ToolEvent.ToolActivateEvent))
                 tool_changed = True
             else:
-                Logger.log("w", "Controller does not have an active tool and could not default to Translate tool.")
+                Logger.log("w", "Controller does not have an active tool and could not default to the tool, called \"{}\".".format(self._fallback_tool))
 
         if tool_changed:
             Selection.setFaceSelectMode(False)
             Selection.clearFace()
             self.activeToolChanged.emit()
 
-    ##  Emitted when the list of tools changes.
     toolsChanged = Signal()
+    """Emitted when the list of tools changes."""
 
-    ##  Emitted when a tool changes its enabled state.
     toolEnabledChanged = Signal()
+    """Emitted when a tool changes its enabled state."""
 
-    ##  Emitted when the active tool changes.
     activeToolChanged = Signal()
+    """Emitted when the active tool changes."""
 
-    ##  Emitted whenever a tool starts a longer operation.
-    #
-    #   \param tool The tool that started the operation.
-    #   \sa Tool::startOperation
     toolOperationStarted = Signal()
+    """Emitted whenever a tool starts a longer operation.
+    
+    :param tool: The tool that started the operation.
+    :sa Tool::startOperation
+    """
 
-    ##  Emitted whenever a tool stops a longer operation.
-    #
-    #   \param tool The tool that stopped the operation.
-    #   \sa Tool::stopOperation
     toolOperationStopped = Signal()
+    """Emitted whenever a tool stops a longer operation.
+    
+    :param tool: The tool that stopped the operation.
+    :sa Tool::stopOperation
+    """
 
-    ##  Get the scene
-    #   \return scene \type{Scene}
     def getScene(self) -> Scene:
+        """Get the scene
+
+        :return: scene
+        """
+
         return self._scene
 
-    ##  Process an event
-    #   \param event \type{Event} event to be handle.
-    #   The event is first passed to the selection tool, then the active tool and finally the camera tool.
-    #   If none of these events handle it (when they return something that does not evaluate to true)
-    #   a context menu signal is emitted.
     def event(self, event: Event):
+        """Process an event
+
+        The event is first passed to the selection tool, then the active tool and finally the camera tool.
+        If none of these events handle it (when they return something that does not evaluate to true)
+        a context menu signal is emitted.
+
+        :param event: event to be handle.
+        """
+
         if self._selection_tool and self._selection_tool.event(event):
             return
 
@@ -345,31 +418,40 @@ class Controller:
 
     contextMenuRequested = Signal()
 
-    ##  Set the tool used for handling camera controls.
-    #   Camera tool is the first tool to receive events.
-    #   The tool can be set by name of the tool or directly passing the tool object.
-    #   \param tool \type{Tool} or \type{string}
-    #   \sa setSelectionTool
-    #   \sa setActiveTool
     def setCameraTool(self, tool: Union["Tool", str]):
+        """Set the tool used for handling camera controls.
+
+        Camera tool is the first tool to receive events.
+        The tool can be set by name of the tool or directly passing the tool object.
+        :param tool:
+        :sa setSelectionTool
+        :sa setActiveTool
+        """
+
         from UM.Tool import Tool
         if isinstance(tool, Tool) or tool is None:
             self._camera_tool = cast(Optional[Tool], tool)
         else:
             self._camera_tool = self.getTool(tool)
 
-    ##  Get the camera tool (if any)
-    #   \returns camera tool (or none)
     def getCameraTool(self) -> Optional["Tool"]:
+        """Get the camera tool (if any)
+
+        :returns: camera tool (or none)
+        """
+
         return self._camera_tool
 
-    ##  Set the tool used for performing selections.
-    #   Selection tool receives its events after camera tool and active tool.
-    #   The tool can be set by name of the tool or directly passing the tool object.
-    #   \param tool \type{Tool} or \type{string}
-    #   \sa setCameraTool
-    #   \sa setActiveTool
     def setSelectionTool(self, tool: Union[str, "Tool"]):
+        """Set the tool used for performing selections.
+
+        Selection tool receives its events after camera tool and active tool.
+        The tool can be set by name of the tool or directly passing the tool object.
+        :param tool:
+        :sa setCameraTool
+        :sa setActiveTool
+        """
+
         from UM.Tool import Tool
         if isinstance(tool, Tool) or tool is None:
             self._selection_tool = cast(Optional[Tool], tool)
@@ -448,13 +530,37 @@ class Controller:
                     camera.setPosition(Vector(0, 100, 700))
                     self._camera_tool.rotateCamera(0, angle)  # type: ignore
 
-    ##  Changes the origin of the camera, i.e. where it looks at.
-    #   \param coordinate One of the following options:
-    #    - "home": The centre of the build plate.
-    #    - "3d": The centre of the build volume.
-    #    - "x", "y" and "z": Also the centre of the build plate. These are just
-    #      aliases for the setCameraRotation function.
+    # Position camera view according to defined position
+    def setCameraPosition(self, x_position: int = 0, y_position: int = 0, z_position: int = 0) -> None:
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        camera.setPosition(Vector(x_position, y_position, z_position))
+
+    # Indicate position where the camera should point at
+    def setLookAtPosition(self, x_look_at_position: int = 0, y_look_at_position: int = 0, z_look_at_position: int = 0) -> None:
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        camera.lookAt(Vector(x_look_at_position, y_look_at_position, z_look_at_position))
+
+    # Sets the zoom factor of the camera
+    def setCameraZoomFactor(self, camera_zoom_factor: float = 0) -> None:
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        camera.setZoomFactor(camera_zoom_factor)
+
     def setCameraOrigin(self, coordinate: str = "home"):
+        """Changes the origin of the camera, i.e. where it looks at.
+
+        :param coordinate: One of the following options:
+        - "home": The centre of the build plate.
+        - "3d": The centre of the build volume.
+        - "x", "y" and "z": Also the centre of the build plate. These are just
+        aliases for the setCameraRotation function.
+        """
+
         camera = self._scene.getActiveCamera()
         if not camera:
             return
@@ -465,4 +571,4 @@ class Controller:
             "y": Vector(0, 100, 0),
             "z": Vector(0, 100, 1)
         }
-        camera.lookAt(coordinates[coordinate])
\ No newline at end of file
+        camera.lookAt(coordinates[coordinate])
