@@ -251,15 +251,16 @@ class Theme(QObject):
             "line": QSizeF(self._em_width, self._em_height)
         }
 
-    ##  Get the singleton instance for this class.
     @classmethod
-    def getInstance(cls, engine = None):
+    def getInstance(cls, engine = None) -> "Theme":
+        """Get the singleton instance for this class."""
+
         # Note: Explicit use of class name to prevent issues with inheritance.
         if Theme.__instance is None:
             Theme.__instance = cls(engine)
         return Theme.__instance
 
-    __instance = None   # type: 'Theme'
+    __instance = None   # type: "Theme"
 
 
 def createTheme(engine, script_engine = None):
